import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import 'package:puzzlegame/cubit/game_cubit.dart';
import 'package:puzzlegame/models/game/game.dart';
import 'package:puzzlegame/ui/widgets/game/game_shuffling.dart';
import 'package:puzzlegame/ui/widgets/game/game_tileset.dart';

class GameBoardWidget extends StatelessWidget {
  const GameBoardWidget({super.key});

  @override
  Widget build(BuildContext context) {
    return Center(
      child: BlocBuilder<GameCubit, GameState>(
        builder: (BuildContext context, GameState gameState) {
          final Game currentGame = gameState.currentGame;

          BlocProvider.of<GameCubit>(context).updateTileSize(
              (MediaQuery.of(context).size.width - 60) /
                  currentGame.gameSettings.tilesetSizeValue);

          if (currentGame.shufflingInProgress) {
            return const GameShufflingWidget();
          } else {
            return const GameTilesetwidget();
          }
        },
      ),
    );
  }
}
